__all__ = [
    'is_exception',
    'webhook_authentication',
    'to_camel_case',
    'data_builder'
]

from .is_exception import is_exception
from .webhook_authentication import webhook_authentication
from .to_camel_case import to_camel_case
from .builders import data_builder



import json

from monopay.exceptions import MonoPayBaseException
from monopay.utils import is_exception
from monopay.utils import data_builder
import requests


def request(url):
    def outer(func):
        def inner(self, *args, **kwargs):
            params = data_builder(**kwargs)

            response = requests.get(
                url=self.__dict__.get(url),
                headers=self.headers,
                params=params
            )

            response_data = response.json()

            if is_exception(response):
                return MonoPayBaseException(
                    err_code=response_data['errCode'],
                    err_text=response_data['errText']
                )

            return func(self, *args, **kwargs, response_data=response_data)

        return inner

    return outer


def post_request(url):
    def outer(func):
        def inner(self, *args, **kwargs):
            data = data_builder(**kwargs)

            print(data)

            response = requests.post(
                url=self.__dict__.get(url),
                headers=self.headers,
                data=json.dumps(data)
            )

            response_data = response.json()

            if is_exception(response):
                return MonoPayBaseException(
                    err_code=response_data['errCode'],
                    err_text=response_data['errText']
                )

            return func(self, *args, **kwargs, response_data=response_data)

        return inner

    return outer



__all__ = [
    'QrListItem',
    'WalletItem',
    'CancelListItem',
    'MerchantStatementItem'
]

from typing import Optional, List
from dataclasses import dataclass


@dataclass
class QrListItem:
    short_qr_id: str
    qr_id: str
    amount_type: str
    page_url: str


@dataclass
class CanceledItem:
    pass


@dataclass
class CancelListItem:
    amount: int
    ccy: int
    date: str
    approval_code: Optional[str]
    rrn: Optional[str]
    masked_pan: str


@dataclass
class MerchantStatementItem:
    invoice_id: str
    status: str
    masked_pan: str
    date: str
    payment_scheme: str
    amount: int
    profit_amount: Optional[int]
    ccy: int
    approval_code: Optional[str]
    rrn: Optional[str]
    reference: Optional[str]
    short_qr_id: Optional[str]
    cancel_list: List[CancelListItem]


@dataclass
class WalletItem:
    card_token: str
    masked_pan: str
    country: Optional[str] = None
